@@ -126,7 +126,7 @@ extern Bool
 RADEONGetATOMTVInfo(xf86OutputPtr output);
 
 extern int
-atombios_external_tmds_setup(xf86OutputPtr output, DisplayModePtr mode);
+atombios_external_tmds_setup(xf86OutputPtr output, int action);
 
 extern void
 atombios_get_command_table_version(atomBiosHandlePtr atomBIOS, int index, int *major, int *minor);
